/*
 *            LVS Windows
 *      The Window Class System
 *
 *        Copyright 1991 (c), Lake View Software
 *                            4321 Harborough Rd.
 *                            Columbus, OH 43220
 *        All rights reserved.
 *
 * DEMO.CPP
 */

#include <stdlib.h>
#include <dos.h>
#include <time.h>
#ifdef __TURBOC__
    #include <alloc.h>
#else
    /*
     * Zortech
     */
    #define gettime         dos_gettime
    #define time            dos_time_t
    #define ti_sec          second
    #define ti_hour         hour
    #define ti_min          minute
    #define random(num)     (int)(((long)rand()*(num))/(0x7FFFU + 1))
    #define delay           usleep
    #define coreleft        farcoreleft
#endif
#include <conio.h>
#include <ctype.h>
#include <stdio.h>
#include "Win_Prn.hpp"	    // print class
#include "WinTools.hpp"     // Add on tools 
#include "WinMenu.hpp"      // Menu class header file
#include "WinPick.hpp"      // Pick-List class header file
#include "WinMouse.Hpp"     // Mouse driver

/*
 * Declaring a single object of this class will save the current
 * screen contents and mode to be restored when the program exits.
 * Note:  - The destructor of this class calls Win_close_all ();
 *        - This needs to be the first declared object so it can
 *          capture the screen.
 *        - This cleans up the loaded VGA fonts and blink state on 
 *          program exit
 */
ScreenState win_state;

/*
 * Some global window objects
 */
WinObj win1,win2;   // Some global window objects, These will be the random windows
WinObj Scroll;	    // Scrolling text demo
WinObj Scroll2;

/*
 * Declare internal functions
 */
int  v_salary(void *num);
int  v_age(void *num);
void rand_open (WinObj &w), show_time (), move_windows (struct time t),
     data_entry (), paradox_entry (), moving_windows (),
     random_windows (), scroll_demo (), scroll_window (),
     menu_demo (), menu_demo2 (), pick_demo (), help (),
     logo ();

int VGA = 0;            // has vga flag
int Use_VGA = 1;        // try to use vga graphics/colors by default

void main (int argc, char **argv)
{
    int arg=0;

    int mouse_mode = M_FULL;              // Default mouse mode 

    while (++arg < argc) 
        {                    // Parse the command line
        if (argv[arg][0] == '-' || argv[arg][0] == '/') 
            {
            switch (toupper (argv[arg][1])) 
                {
                case 'B': 
                    Win_bios(W_ON);     // Turn on use of bios writes
                    break;

                case 'S': 
                    Win_snow(W_ON);     // Turn on use of snow suppression
                    break;

                case 'M': 
                    switch (toupper (argv[arg][2]))
                        {
                        case 'N':                 // Mouse off
                            mouse_mode = M_NONE;
                            break;

                        case 'S':                 // Mouse full support
                            mouse_mode = M_SIMPLE;
                            break;

                        default:
                            Win_mono(W_ON);     // Force black & white only
                            break;
                        }
                        break;
                case '5': 
                    VGA_setlines (50);
                    break;
                case '2':
                    VGA_setlines (28);
                    break;
                case 'V':
                    Use_VGA = 0;
                    break;
                }
            }
        }

    srand (1);

    // Turn clock on
    Win_setbkloop (show_time);          // Set background keyboard loop
                                        // This will run the clock and move win1
                                        // & win2

    Win_setonkey (F1,help);             // Set F1 to function help
	Win_setonkey (F10, Win_Calculator);
	Win_setonkey (F5, Win_Calendar);

    /*
     * Display the logo
     */
    logo ();

    /*
     * Open the main menu
     */
    WinMenu m;  					                // create Main menu object

    int bkground_color = _RED;
    if (VGA)
        bkground_color = _LBLUE;

    m.open (4, 26, 15, 52, W_DOUSIN, bkground_color|WHITE, 
            bkground_color|BLACK);             // Open the window
	m.shadow ();                               // make a shadow for the window
    m.set_keyattr (bkground_color|YELLOW);     // change the hot key attribute

    bkground_color = _LGREY;
    if (VGA)
        bkground_color = _WHITE;
    m.title (" Demo Menu ");                 // Give the window a title

    /*
     * Define the menu selections
     */
    m.prompt (1, 2, 1, " 1. Data entry       ", "", '1');
    m.prompt (2, 2, 2, " 2. Moving windows   ", "", '2');
    m.prompt (3, 2, 3, " 3. Random windows   ", "", '3');
    m.prompt (4, 2, 4, " 4. Lotus style menu ", "", '4');
    m.prompt (5, 2, 5, " 5. Pick-List demo   ", "", '5');
	m.prompt (6, 2, 6, " 6. Scrolling demo   ", "", '6');
    #ifdef __TURBOC__
        m.prompt (7, 2, 7, " 7. Paradox entry    ", "", '7');
    #else
        m.printf (7, 2, " 7. Paradox entry    ");
    #endif
	m.prompt (8, 2, 0, " 0. Quit             ", "", '0');

    /*
     * Init the mouse driver
     */
    Win_mouse_mode (mouse_mode);

    
    int choice = 1;                         // start with menu selection ret_val = 1

    while (choice != 0) 
        {                   // loop till Quit is selected

        choice = m.read (choice);           // get menu selection from user

        switch (choice) 
            {
            case 1: 
                data_entry ();
                break;
            case 2: 
                moving_windows ();
                break;
            case 3: 
                random_windows ();
                break;
            case 4: 
                m.hide ();                  // hide the main menu
                menu_demo ();
                break;
            case 5: 
                pick_demo ();
                break;
			case 6: 
                scroll_demo ();
				break;
            case 7:
                paradox_entry ();
                break;
            }
        }

    // all windows will close automatically with there destructors
}

//*********************Pick-List Demo************************

static char *list[] = { "January",   "February",
                        "March",     "April",
                        "May",       "June",
                        "July",      "August",
                        "September", "October",
                        "November",  "December",  0 };

void pick_demo()
{
    WinPick pl;

    int bk = _BLUE;
    if (VGA)
        bk = _DGREY;

    pl.open (3, 33, 9, 45, W_DOUSIN, bk|WHITE, bk|GREEN);  // open pick window
    pl.shadow();
    pl.title ("Pick-List");
    pl.prompt (list);           // assign a list
    int choice = pl.read();     // get user selection, closes window when finished

    // Display the results of the pick.
    if (choice != W_ESCAPE) 
        {
        WinObj w;
        w.open (6, 23, 11, 53, W_DOUSIN, bk|GREEN, bk|WHITE);
        w.title (" Answer ");
        w.shadow ();
        w.printf (1, 1, "You picked ");
        w.set_attr (bk|YELLOW);
        w.printf ("%s.", list[choice]);
        w.set_attr (bk|LGREY);
        w.printf (3, 1, "Press any key to continue.");
        w.get_key ();
        }
}

//**********************Menu Demo****************************
// First level of a lotus style menu
void menu_demo ()
{
    WinMenu m;                                      // Create a menu object
    m.open (1, 0, 1, 79, W_NOBORDER, _BLUE|WHITE);  // init the window
    m.message_line (2);                             // Set message line to 2
    m.set_keyattr (_BLUE|YELLOW);                   // hot key attribute
    m.set_barattr (_RED|WHITE);                     // make bar white on red

    // Declare some menu selections
    m.prompt (0,  0, 1, " Worksheet ", "Global, Column, Status, Delete", 'W');
    m.prompt (0, 12, 2, " Range ", "Format, Label, Erase, Name, Justify, Prot", 'R');
    m.prompt (0, 20, 3, " Copy ",  "Copy a cell or range of cells", 'C');
    m.prompt (0, 27, 4, " mOve ",  "Move a cell or range of cells", 'O');
    m.prompt (0, 34, 5, " File ",  "Retrieve, Save, Combine, Xtract...", 'F');
    m.prompt (0, 41, 6, " Print ", "Print a worksheet to a printer or file", 'P');
    m.prompt (0, 49, 0, " Quit ",  "Quit the menu demo", 'Q');
                    
    int choice=1;
    while (choice != 0) 
        {
        choice = m.read(choice);        // Get a menu selection
		if (choice == W_ESCAPE)
			break;
        switch (choice) 
            {
            case 1:
                menu_demo2();           // Goto next menu level
                break;
            case 2:
                break;
            }
        }
    // Menu object will destruct and close automatically.
}

void menu_demo2()       // Second level of a lotus style menu
{
    WinMenu m;
    m.open (1, 0, 1, 79, W_NOBORDER, _BLUE|WHITE);
    m.message_line (2);                             // Set message line to 1
    m.set_keyattr (_BLUE|YELLOW);                   // make hot key yellow
    m.set_barattr (_RED|WHITE);                     // make bar white on red

    m.prompt (0,  0, 1, " Global ", "Global functions", 'G');
    m.prompt (0,  9, 2, " Column ", "Delete, Insert", 'C');
    m.prompt (0, 18, 3, " Status ", "Display worksheet status", 'S');
    m.prompt (0, 27, 4, " Delete ", "Delete the entire worksheet", 'D');
    m.prompt (0, 36, 5, " sYstem ", "Shell to DOS", 'Y');
    m.prompt (0, 45, 0, " Quit ", "Quit to the main menu", 'Q');
    int choice=1;
    while (choice != 0) 
        {
        choice = m.read(choice);
		if (choice == W_ESCAPE)
			break;

        switch (choice) 
            {
            case 1:
                break;
            case 2:
                break;
            case 5:
                Win_clrscr ();
                puts ("Type 'EXIT' to return to demo.\n\n\n");
                system ("");
                m.update_all ();
                break;
            }
        }
}

//*************************Moving windows Demo*************************
// Open the moving windows
void moving_windows()
{
	// Windows will open or close with each call
    if (!win1.is_open()) 
        {
        win1.open (1, 1, 10, 25, W_DOUBLE, _MAGENTA|BLACK, _MAGENTA|BLUE);
        win1.shadow ();
        win1.title (" Moving Window 1 ");
        win1 << "Here is some text in a moving window.";

        win2.open (1, 54, 10, 79, W_DOUSIN, _GREEN|BLACK, _GREEN|WHITE);
        win2.shadow ();
        win2.title (" Moving Window 2 ");
        win2 << "Here is some text in the second moving window.";
        }
    else 
        {
        win1.close ();
        win2.close ();
        }
}

// Function to move the random windows every second
// This function is called by the clock function with the current time
void move_windows (struct time t)
{
    static int x = 1, y = 1, sec_hold = 0;

    if (!win1.is_open())
        return;

    if (sec_hold != t.ti_sec)       // only move once a second
        {
        sec_hold = t.ti_sec;
        x += 6;
        y += 2;
        if ( x > 60 || y > 18)
            {
            x = 0;
            y = 0;
            }
        win1.move (y, x);

        // randomly move the second window
        win2.move (random(Win_Max_y () - 11), random(54));
        }
}

//***********************Scroll Demo**********************************
void scroll_demo ()
{
    // Windows will open or close with each call
    if (!Scroll.is_open()) 
        {
        int bkground = _CYAN;
        if (VGA)
            bkground = _LCYAN;

        Scroll.open (2, 56, 8, 78, W_DOUSIN, bkground|BLACK, bkground|BLUE);
        Scroll.shadow ();
        Scroll.title (" Slow Scroll ");

        if (VGA)
            bkground = _YELLOW;
        else
            bkground = _BLUE;

		Scroll2.open (11, 56, 16, 78, W_DOUSIN, bkground|RED, bkground|BLACK);
		Scroll2.shadow ();
		Scroll2.title (" Fast Scroll ");
        }
    else 
        {
        Scroll.close ();
		Scroll2.close ();
        }
}

void random_message (WinObj &w)
/*
 * All the print messages here accomplish the same results,
 * a cr/lf followed by some text.
 */
{
	switch (random (6))
		{
	case 0:
		w << '\n' << ' ' << w.get_version();
		break;
	case 1:
		w << '\n' << " Lake View Software";
		break;
	case 2:
		w << '\n' << " Borland C++ v2.0";
		break;
	case 3:
		w << "\n Turbo C++ v1.01";
		break;
	case 4:
		w.printf ("\n Support Shareware");
		break;
	default:
		w.puts ("\n Some Scrolling Text");
		break;
	}
}

void scroll_window ()
{
    if (!Scroll.is_open())
        return;

   	static clock_t clock1 = 0, clock2 = 0;

	if ((clock1 + 5) < clock () )
		{
		random_message (Scroll);
        clock1 = clock ();
		}

	if ((clock2+1) < clock ())
		{
		random_message (Scroll2);	// Fast scroll
		clock2 = clock ();
		}
}

//**********************Random Windows Demo***************************
void random_windows ()
{
    WinObj win1, win2, win3, win4, win5;    // Create some random windows
    rand_open (win1);
    delay (250);         // Pause for effect
    rand_open (win2);
    delay (250);         // Pause for effect
    rand_open (win3);
    delay (250);         // Pause for effect
    rand_open (win4);
    delay (250);         // Pause for effect
    rand_open (win5);
    delay (250);         // Pause for effect
    win1.clr_win ();
    win1 << "Press any key to continue.";
	win1.get_key (0, 10);	// wait 10 seconds for a key
}

// Function to open random windows on the screen
void rand_open (WinObj &w)
{
    static cnt = 0;
    int sr, er, sc, ec;
    int color, bcolor, btype;

    sr = random (Win_Max_y () - 14);
    sc = random (40);
    er = sr + 10 + random ((Win_Max_y () - 14)-sr);
    ec = sc + 15 + random (63-sc);

    int bk_colors = 8;
    if (VGA)
        bk_colors = 16;

    color = random (bk_colors) * 16 + random (16);
    bcolor = random (bk_colors) * 16 + random (16);
    btype = random (MAX_BOX_TYPE + 1);
    w.open (sr, sc, er, ec, btype, color, bcolor);
    w.shadow ();
    w.title (" Random Window ");
    w << "This is random window #(" << ++cnt << ")";
}

//************************Data Entry Demo********************************

void paint_data_screen (WinObj &w)
{
    w.printf (1, 1, "  Last Name:");
    w.printf (2, 1, " First Name:");
    w.printf (4, 1, "    Address:");
    w.printf (6, 1, "City/St/Zip:");
    w.printf (6, 29, ",");
    w.printf (8, 1, "      Phone:");
    w.printf (10, 1, " Start Date:");
    w.printf (12, 1, "       Age:");
    w.printf (14, 1, "    Salary:");
}

int read_data (WinObj &w)		// read data for data_entry or print for print_data
{
	// Declare data variables.
    static char lastname[16] = "", firstname[16] = "";
    static char add1[31] = "", add2[31] = "";
    static char city[16] = "", state[3] = "", zip[11] = "";
    static char phone[15] = "";
    static char start[9] = "";
    static double salary = 0;
    static long age = 0;

    str_setsz (lastname, 15);    // We won't use a picture for these
    str_setsz (firstname, 15);   // variables, so we must give the initial
    str_setsz (add1, 30);        // value some size.
    str_setsz (add2, 30);
    str_setsz (city, 15);
    str_setsz (state, 2);

    w.get (1, 14, lastname, 'F');  // force first character of these fields
    w.get (2, 14, firstname, 'F'); // to be upper case with a 'F'orce style.
    w.get (4, 14, add1, 'F');
    w.get (5, 14, add2, 'F');
    w.get (6, 14, city, 'F');

    w.get (6, 31, state, 'U');        // force state to be uppercase
    w.get (6, 35, zip, "99999-9999");  // use a picture for a zip code

    w.get (8, 14, phone, 'P');     // get a phone style
    w.get (10, 14, start, 'D');    // get a date style
    w.get (12, 14, age, 3);        // Limit age to 3 places
    w.valid (v_age);            // check age to be over 15

    w.get (14, 14, salary, 9);     // limit salary to hundred thousands
    w.valid (v_salary);         // check to be sure salary is over $10,000

    return (w.read ());           // read the gets
}

void print_data ()
{
	if (!Win_Get_YN ("Printer ready"))
		return;
	
	Win_Prn p;
	if (p.open () != W_OK)
		{
		Win_Error ("Can't open printer port!");
		return;
		}

	paint_data_screen (p);		// Paint the screen in memory
	read_data (p);				// Paint the gets in memory

	//
	// Now lets change some of the print attributes.
	// These normaly would could be set before each printf or puts...
	// but since I used the same routines to print the screen and the
	// page this is being done later.
	//
	p.set_pageattr (P_NORMAL);	// Set attributes for the entire page
	p.set_attr (P_UNDERLINE, 1, 14, 15);	// Underline the last name

	p.print_page ();			// Print physical page

	p.close ();					// could let close with distructor
}

void data_entry()
{
    WinObj  w(40,65);      // Create the main entry window
    w.open (3, 14, 12, 62, W_DOUBLE, _BLUE|WHITE, _BLUE|YELLOW);
    w.shadow ();
    w.title(" Data Entry ");

	paint_data_screen (w);		// Paint the screen
    if (read_data (w) != ESC)				// Get the data
    	if (Win_Get_YN ("Do you wish to print the data"))
	    	print_data ();
}

int v_salary(void *num)             // Valid function for salary
{
    if (* (double *)num >= 10000)     // Must first conver num to a double type
        return 1;                   // pointer.  if >= 10,000 return true.

	Win_Error (" Salary must be > 10,000! ");
    return 0;                       // return false.
}

int v_age(void *num)             // Valid function for age
{
    if (* (int *)num > 15)     		// Must first conver age to a integer type
        return 1;                   // pointer.  if > 15 return true.

	Win_Error (" Age must be > 15 ");
    return 0;                       // return false.
}

//*************************Paradox Entry*********************************
#ifdef __TURBOC__
char *Text[] = {
    "Entering text in a Paradox Table is one of the many features of LVS",
    "Windows.  It allows you to enter directly into the Paradox record",
    "buffer, with no temporary variables.",
    "",
    "I'd like to show you how to enter text in a Paradox Table using LVS",
    "Windows, but I would have to write this in the large model and I've",
    "only supplied the small model here.  So here's what the code would",
    "look like.",
    "",
    "To view the code use the Up and Down arrows keys.",
    "                   Esc to exit",
    "",
    "",
    "#include <pxengine.h>",
    "#include <winobj.hpp>",
    "",
    "main ()  {",
    "   TABLEHANDLE TblHandle;",
    "   RECORDHANDLE RecHandle;",
    "",
    "   PXInit ();                              // Init the PX engine",
    "   PXTblOpen (\"SALES\", &TblHandle, 0, 0);  // Open the table",
    "   PXRecBufOpen (TblHandle, &RecHandle);   // Open a record buffer",
	"   PXRecFirst (TblHandle);                 // Point to the first record",
    "   PXRecGet (TblHandle, RecHandle);        // Read the record.",
    "",
    "   WinObj w;                               // my window object",
    "   w.open (0, 0, 24, 79);                  // Open the window",
    "   w.PXrecord (TblHandle, RecHandle);      // give winobj the PX info",
    "   w.PXget (1, 1, 1, 'F');                 // Get the first field, style 'F'",
    "   w.PXget (3, 1, 2, 'F');",
    "   w.PXget (5, 1, 11, 'P');                // Get the 11 field, style Phone",
    "   w.PXget (7, 1, 17, 10, 2);",
    "   w.PXget (9, 1, 18, 3, 0);",
    "   w.PXread ();                            // Read the user input",
    "   PXRecUpdate (TblHandle, RecHandle);     // update the PX record",
    "   PXExit ();                              // Stop the PX engine",
    "}",
    "",
    "",
    "Paradox in a Trademark of Borland",
    0 };
#endif

void paradox_entry ()
{
#ifdef __TURBOC__
    WinObj w (43, 78);

    w.open (1, 0, 23, 79, W_SINGLE, _BLUE|WHITE, _BLUE|CYAN);
    w.title (" Paradox Entry ");
    int i = -1;
    while (Text [++i])
        w.puts (i+1, 2, Text [i]);

    w.gotoxy ();

    int key = 0;
    do {
        key = w.get_key (0, 60);      // wait 1 minute for a key press

        switch (key)
            {
            case UP:
                w.gotoxy (w.wherey () - 1, w.wherex ());
                break;
            case DOWN:
                if (w.wherey () < w.max_y ())
                    w.gotoxy (w.wherey () + 1, w.wherex ());
                break;
            case LEFT:
                w.gotoxy (w.wherey (), w.wherex () - 1);
                break;
            case RIGHT:
                w.gotoxy (w.wherey (), w.wherex () + 1);
                break;
            }

        } while (key && (key != ESC));
#endif
}

//**************************Clock function*******************************

void show_time ()        // Show time in upper left corner of screen
{
    static struct time t;
    static WinObj clock_win;    // create a static window object
    static unsigned char hold;

    WinObj *w_hold = Win_whos_active ();

    if (!clock_win.is_open())       // open small clock window once
		{
        int color = _LGREY|BLUE;
        if (VGA)
            color = _WHITE|BLUE;
            
        clock_win.open (0, 0, 0, 79, W_NOBORDER, color);
		clock_win.center (0, "Lake View Software");
		}

    /*
     * First show the clock
     */
    gettime (&t);
    if (hold != t.ti_sec) {
        hold = t.ti_sec;
        clock_win.printf (0, 1, "%02d:%02d:%02d", t.ti_hour, t.ti_min, t.ti_sec);
        #ifndef DOS386
        clock_win.printf (0, 73, "%6ld", (long )coreleft ());  // show free mem
        #endif
    }

    /*
     * Move the random windows
     */
    move_windows(t);

    /*
     * scroll text in window
     */
	scroll_window ();

    /*
     * Restore the currently active window
     */
    w_hold->set_active ();
}

//**********************Help Function***********************************
void help()
{
    static int is_active=0;
    int old_x, old_y;
    if (is_active)
        return;
    is_active++;
    Win_getxy (&old_y, &old_x);       // Save current cursor position

    WinObj help;

    int bkground = _LGREY;
    if (VGA)
        bkground = _LBLUE;

    help.open (2, 11, 22, 67, W_SINGLE, bkground|BLACK, bkground|WHITE);
    help.shadow ();
    help.title (" Help Window ");
    help.center (1, "Data Entry Keys");
    help.gotoxy (3, 1) << "/ arrows        Move cursor to next/prev field.";
    help.printf (4, 1, "%c/%c arrows        Move cursor to next/prev column.",26,27);
    help.printf (5, 1, "Ctrl_%c/%c arrows   Pan window right/left one column.",26,27);
    help.printf (6, 1, "Ctrl_end/home     Pan window down/up one row.");
    help.printf (7, 1, "PgDn              Exit read mode.");
    help.printf (8, 1, "Esc               Quit read mode.");
    int hold_attr = help.set_attr (bkground|WHITE);
    help.center (10, "Non Picture Gets Only");
    help.set_attr (hold_attr);
    help.printf (11, 1, "Insert            Toggle insert/overtype mode.");
    help.printf (12, 1, "Delete            Delete character under cursor.");
    help.printf (13, 1, "Backspace         Delete previous character.");
    hold_attr = help.set_attr (bkground|RED);
    help.center (17, "Copyright (c) 1991, Lake View Software");
    help.set_attr (hold_attr);
    help.center (18, "Press any key to exit.");
	Win_getkey (15);				// wait 15 seconds for a key 

    Win_gotoxy (old_y, old_x);        // Restore cursor position
    is_active = 0;                  // turn off active flag
}


//*****************************Logo*****************************************

void function_key_mouse_routine (int y, int x)
/*
 * Check mouse clicks for function keys
 * This routine will be attached to the footer
 * window.
 */
{
    if (y != 0)
        return;

    if (x >=21 && x <= 27)
        Win_keystuf (F1);

    if (x >= 30 && x <= 40)
        Win_keystuf (F5);

    if (x >= 43 && x <= 56)
        Win_keystuf (F10);
}

void logo ()
{
    int max_y = Win_Max_y ();           // Get the max y coordinate

    if (Use_VGA)
        {
        if (VGA_videotype () == V_VGA)
            {
            /*
             * Turn blink off to allow highlight backgrounds
             */
            VGA_setblink (W_OFF);

            /*
             * load special fonts
             */

            VGA_fontload ();

            VGA++;      // set vga flag
            }
        }

    /*
     * Open a window to cover the screen
     */
	static WinObj Main;
    int bkground = _BLACK;
    if (VGA)
        bkground = _DGREY;
	Main.open (0, 0, max_y, 79, W_NOBORDER, bkground|LGREY);

	for (int x = 0; x < max_y * 2 + max_y / 5; x++)
		Main.printf ("- LVS Windows -- Lake View Software -");
	

    /*
     * Display the footer
     */
	static WinObj Footer;
    int color = _LGREY|BLACK;
    if (VGA)
        color = _WHITE|BLACK;

	Footer.open (max_y, 0, max_y, 79, W_NOBORDER, color);
	Footer.puts (0, 21, "F1=Help, F5=Calendar, F10=Calculator");
    Footer.set_mouseroutine (function_key_mouse_routine);

    /*
     * Display the logo
     */
    static WinObj logo;
    color = _BLACK;
    if (VGA)
        color = _DGREY;

    logo.open (max_y - 5, 19, max_y - 2, 59, W_DOUSIN, 
               color|WHITE, color|YELLOW);
    logo.shadow ();

    logo.title (logo.get_version ());
    logo.center (0, "The C++ window class system");
    logo.center (1,"Copyright (c) 1991");

    show_time ();       // displays the title bar and time for the first time
}
